<?php
if (!defined('_PS_VERSION_')) {
    exit;
}
/**
 *
 * @version 1.0
 * @author nalic
 */
class OrderStatus
{
    const AWATING_CHECK_PAYMENT = 1;
    const PAYMENT_ACCEPTED = 2;
    const PROCESSING_IN_PROGRESS = 3;
    const SHIPPED = 4;
    const DELIVERED = 5;
    const CANCELED = 6;
    const REFUNDED = 7;
    const PAYMENT_ERROR = 8;
    const ON_BACKORDER_PAID = 9;
}

class GpnDataOrderStatus
{
    public static $DECLINED = array(
        'name' => 'GPNDATA_DECLINE_STATUS',
        'value' => 'GPN DATA Declined',
    );

    public static $CHARGEBACK = array(
        'name' => 'GPNDATA_CHARGEBACK_STATUS',
        'value' => 'GPN DATA Chargeback',
    );

    public static $PARTIAL_CAPTURED = array(
        'name' => 'GPNDATA_PARTIAL_CAPTURED_STATUS',
        'value' => 'GPN DATA Partial Captured',
    );

    public static $PARTIAL_REFUNDED = array(
        'name' => 'GPNDATA_PARTIAL_REFUNDED_STATUS',
        'value' => 'GPN DATA Partial Refunded',
    );

    public static $PENDING_STAFF_APPROVAL = array(
        'name' => 'GPNDATA_PENDING_STAFF_APPROVAL_STATUS',
        'value' => 'GPN DATA Pending staff approval',
    );

    public static function getOrderStateDeclinedId()
    {
        return intval(Configuration::get(GpnDataOrderStatus::$DECLINED['name']));
    }

    public static function getOrderStateChargebackId()
    {
        return intval(Configuration::get(GpnDataOrderStatus::$CHARGEBACK['name']));
    }

    public static function getOrderStatePartialCapturedId()
    {
        return intval(Configuration::get(GpnDataOrderStatus::$PARTIAL_CAPTURED['name']));
    }

    public static function getOrderStatePartialRefundedId()
    {
        return intval(Configuration::get(GpnDataOrderStatus::$PARTIAL_REFUNDED['name']));
    }

    public static function getOrderStatePendingStaffApprovalId()
    {
        return intval(Configuration::get(GpnDataOrderStatus::$PENDING_STAFF_APPROVAL['name']));
    }

    public static function setOrderNewStatus($id_order, $id_status)
    {
        $obj_order = new Order($id_order);
        $id_current_state = $obj_order->getCurrentOrderState()->id;
        if (empty($id_current_state)) {
            return 'Order was not found, unable to continue with state change. Please update the Order status manually!';
        }
        if ($id_current_state !== $id_status) {
            $history = new OrderHistory();
            $history->id_order = (int) $obj_order->id;
            $history->changeIdOrderState($id_status, (int) ($obj_order->id));
            $history->add();
        }
        return true;
    }

    /**
     * Summary of addOrderState
     * @param string $config_name - Name of the status in PS config
     * @param string Name
     * @param bool Send an e-mail to customer ?
     * @param string $module_name
     * @param bool $invoice Allow customer to view and download invoice when order is at this state
     * @param string $color Display state in the specified color
     * @param bool $logable Log authorization
     * @param bool $paid Paid
     * @param bool $pdf_invoice Attach PDF Invoice
     * @return boolean
     */
    public static function addOrderState($config_name, $name, $send_email, $module_name, $invoice, $color, $logable, $paid, $pdf_invoice, $template = '')
    {
        // Create the order state, if the state does not exist in configuration
        if (empty(Configuration::get($config_name))) {
            // create new order state
            $order_state = new OrderState();
            $order_state->name = array();
            $order_state->color = $color;
            $order_state->send_email = $send_email;
            $order_state->module_name = $module_name;
            $order_state->invoice = $invoice;
            $order_state->color = $color;
            $order_state->logable = $logable;
            $order_state->paid = $paid;
            $order_state->pdf_invoice = $pdf_invoice;
            $order_state->template = $template;
            $languages = Language::getLanguages(false);
            foreach ($languages as $language) {
                $order_state->name[$language['id_lang']] = $name;
            }
            // Update object
            $result = $order_state->add();
            if ($result) {
                Configuration::updateValue("$config_name", pSQL($order_state->id));
            }
        }
        return true;
    }
}
