<?php

if(!defined('_PS_VERSION_')){
	exit;
}
include_once(_PS_MODULE_DIR_ .'/gpndata/library/gpndata.php');
/**
 * @version 1.0
 * @author nalic
 */
class GpnDataGatewayOperationsModuleFrontController extends ModuleFrontController
{
    private $_API;
    private $db;
    public function initContent(){

        parent::initContent();
        if(Tools::getValue('id_order') && Tools::getValue('transaction_action')){
            $module = new GpnData();
            $this->db = $module->db;
            $this->_API = $module->_API;
            $transaction = $this->db->getTransactionByOrderId(Tools::getValue('id_order'), true);
            if(empty($transaction['transaction']['gate_trans_id'])){
                $transaction_data = $this->_API->getTransactionDetails($transaction['transaction']['merchant_trans_id']);
                $transaction['transaction']['gate_trans_id'] = strval($transaction_data->transaction->gatetransid);
                $this->db->updateTransactionByTransactionId($transaction['transaction']);
            }
            $transaction['action'] = Tools::getValue('transaction_action');
            $transaction['amount'] = Tools::getValue('transaction_amount');
            $transaction['reason'] = Tools::getValue('refund_reason');
            $transaction_amount = sprintf('%.2F',str_replace(',','', $transaction['transaction']['payment_amount']));
            $transaction['amount'] =  sprintf('%.2F', str_replace(',','', $transaction['amount']));
            $is_full_transaction_amount = $transaction_amount == $transaction['amount'];
            $transaction = $this->_perfromTransactionAction($is_full_transaction_amount, $transaction);

            $json = array(
               'status' => isset($transaction['error']) ? 'error' : 'success',
               'message' => isset($transaction['error']) ? $transaction['error'] : ''
            );

        } else
            $json = array(
                'status' => 'error',
                'message' => 'Error when while performing action.'
            );

        header('Content-type: application/json; charset=utf-8');
        die(Tools::jsonEncode($json));
    }

    private function _perfromTransactionAction($is_full_transaction_amount, $transaction) {
        try
        {
            $payment_amount = $transaction['transaction']['payment_amount'];
            $api_request_result = null;
            $adjustment_data = array(
                          'gate_trans_id' => $transaction['transaction']['gate_trans_id'],
                          'amount' => $transaction['amount']
                          );
            $transaction_data = array('gate_trans_id' => $transaction['transaction']['gate_trans_id']);
            switch ($transaction['action']){
                case 'capture':
                    $api_request_result = $this->_API->requestCaptureAuthorization($transaction['transaction']['gate_trans_id'], $transaction['amount']);
                    $adjustment_data['additional_text'] = $is_full_transaction_amount ? 'Captured' : 'Payment was partitialy captured. Original transaction amount: '.$transaction['transaction']['payment_amount'].' Captured amount: '.$transaction['amount'];
                    $transaction_data['is_captured'] = 1;
                    $transaction_data['payment_amount'] = (floatval($transaction['transaction']['payment_amount']) - (floatval($transaction['transaction']['payment_amount']) - floatval($transaction['amount'])));
                    break;
                case 'refund':
                    $api_request_result = $this->_API->refundTransaction($transaction['transaction']['gate_trans_id'], $transaction['amount'], $transaction['reason']);
                    $adjustment_data['additional_text'] = $is_full_transaction_amount ? 'Refunded' : 'Payment was partitialy refunded. Partial refund amount: '.$transaction['amount'];
                    $transaction_data['is_refunded'] = 1;
                    $transaction_data['payment_amount'] = (floatval($transaction['transaction']['payment_amount']) - floatval($transaction['amount']));
                    break;
                case 'void':
                    $api_request_result = $this->_API->requestCancelAuthorization($transaction['transaction']['gate_trans_id']);
                    $transaction_data['is_voided'] = 1;
                    break;
                default:
            }

            $result = $api_request_result->getTransactionResult();
            //if it was error append it here and return transaction result
            if($result == TransactionStatus::ERROR){
                $transaction['error'] = 'Error occured while perfoming action.\n\n'. $api_request_result->getErrorMsg();
                return $transaction;
            }
            $order_status_update_result = false;
            if($result == TransactionStatus::SUCCESS){
                $adjustment_data['description'] = strval($api_request_result->getResponseValue('description'));
                if($transaction['action'] === 'refund'){
                    $order_status_update_result = GpnDataOrderStatus::setOrderNewStatus($transaction['transaction']['order_id'], $is_full_transaction_amount ? OrderStatus::REFUNDED : GpnDataOrderStatus::getOrderStatePartialRefundedId());
                }
                if($transaction['action'] === 'capture'){
                    $order_status_update_result = GpnDataOrderStatus::setOrderNewStatus($transaction['transaction']['order_id'], $is_full_transaction_amount ? OrderStatus::PAYMENT_ACCEPTED : GpnDataOrderStatus::getOrderStatePartialCapturedId());
                }
            }
            if ($result == TransactionStatus::CANCELED){
                $order_status_update_result = GpnDataOrderStatus::setOrderNewStatus($transaction['transaction']['order_id'], OrderStatus::CANCELED);
            }
            if ($result == TransactionStatus::REFUND_PENDING || $result == TransactionStatus::REQUEST_PENDING ||
                $result == TransactionStatus::PENDING){
                $adjustment_data['description'] = strval($api_request_result->getResponseValue('description'));
                $adjustment_data['additional_text'] = 'Pending on action: '.strtoupper($transaction['action']). ' for amount: '.$transaction['amount'];
                $transaction_data['is_refund_pending'] = ($result == TransactionStatus::REQUEST_PENDING && $transaction['action'] == 'refund' ? 1: 0);
                $transaction_data['payment_amount'] = $payment_amount;
            }
            if(!is_bool($order_status_update_result)){
                $transaction['error'] = $order_status_update_result;
                return $transaction;
            }
            $adjustment_data['status'] = $result;
            $adjustment_data['state'] = $result;

            $this->db->addTransactionAdjustmentFromArray($adjustment_data);
            $this->db->updateTransaction($transaction_data);
        }
        catch (Exception $exception)
        {
            $transaction['error'] = $exception->getMessage();
        }
        return $transaction;
    }
}